@@ -123,8 +123,9 @@ def call_method(client, args, unknown_args, wait_thread):
 
     while True:
         view_params = get_view_params(client, method + '_view', step=None,
-                                      expert=True,
-                                      dispatch_usenew=dispatch_usenew)
+                                      expert=True, 
+                                      dispatch_usenew=dispatch_usenew,
+                                      unknown_args=unknown_args)
 
         view = get_view(client, method, client.sid, view_params)
         method_parser = get_method_argparser(view, args)
